local ldb_tip
local frame   = CreateFrame("Button", "The Traveller's Reference")
local tourist = LibStub("LibTourist-3.0")
local ldb_obj = LibStub("LibDataBroker-1.1"):NewDataObject("The Traveller's Reference", {
	icon     = "Interface\\Icons\\INV_Misc_Note_01.png",
	label    = "The Traveller's Reference"
})

TheTravellersReferenceDB = TheTravellersReferenceDB or { full = true, range = true, drag = false }

local function get_zone_info(zone)
	local continent            = zone and (tourist:GetContinent(zone) or "Unknown")
	zone                       = zone or GetRealZoneText()
	local min_level, max_level = tourist:GetLevel(zone)
	min_level                  = min_level == 0 and 1 or min_level
	max_level                  = max_level == 0 and (select(4, GetBuildInfo()) >= 30000 and 80 or 70) or max_level
	local red, green, blue     = tourist:GetLevelColor(zone)
	local color                = ("|cff%02x%02x%02x"):format(red * 255, green * 255, blue * 255)
	local range                = min_level.."-"..max_level

	return color..range.."|r", continent or zone
end

local function iterate_zones(tip, which)
	local _, zones = tourist["IterateRecommended"..which](tourist)

	if type(zones) == "table" then
		tip:AddLine("|cff8888ee"..which.." within your level range:|r")

		for zone in pairs(zones) do
			local range, continent = get_zone_info(zone)
			local complex          = tourist:GetComplex(zone)
			zone                   = which == "Instances" and (complex and complex.." " or "")..zone.."  ("..(tourist:GetInstanceZone(zone) or "Unknown")..")" or zone

			tip:AddDoubleLine(range.."|cffeeeeee - "..zone.."|r", "|cffeeeeee"..continent.."|r")
		end
	end
end

local function set_ldb_obj_text()
	local range, zone = get_zone_info()
	local minimap     = GetMinimapZoneText()
	zone              = TheTravellersReferenceDB.full and (minimap and zone.." ("..minimap..")" or zone) or (minimap or zone)
	ldb_obj.text      = zone.."|r"..(TheTravellersReferenceDB.range and " ["..range.."]" or "")
end

function ldb_obj.OnTooltipShow(tip)
	if not ldb_tip then ldb_tip = tip end

	tip:ClearLines()

	tip:AddLine("The Traveller's Reference: A DataBroker Plugin")
	tip:AddLine(" ")

	if TheTravellersReferenceDB.drag then
		tip:AddLine("|cff69b950Control + Click:|r |cffeeeeeeToggles plugin-dragging mode.|r")

		tip:Show()

		return
	end

	local _, borders = tourist:IterateBorderZones(GetRealZoneText())

	if type(borders) == "table" then
		tip:AddLine("|cff8888eeZones bordering this one:|r")

		for zone in pairs(borders) do
			local range, continent = get_zone_info(zone)
	
			tip:AddDoubleLine(range.."|cffeeeeee - "..zone.."|r", "|cffeeeeee"..continent.."|r")
		end
	
		tip:AddLine(" ")
	end

	iterate_zones(tip, "Instances")
	tip:AddLine(" ")
	iterate_zones(tip, "Zones")

	local range, zone = get_zone_info()

	tip:AddLine(" ")
	tip:AddDoubleLine("|cffffd700The level range of "..zone.." is:|r", range)
	tip:AddLine(" ")
	tip:AddLine("|cff69b950Click:|r |cffeeeeeeToggles the World Map open/closed.|r")
	tip:AddLine("|cff69b950Alt + Click:|r |cffeeeeeeToggles the zone/sub-zone display style.|r")
	tip:AddLine("|cff69b950Shift + Click:|r |cffeeeeeeToggles showing the range on the display.|r")
	tip:AddLine("|cff69b950Control + Click:|r |cffeeeeeeToggles plugin-dragging mode.|r")


	tip:Show()
end

function ldb_obj.OnClick()
	if IsControlKeyDown() then
		TheTravellersReferenceDB.drag = not TheTravellersReferenceDB.drag

		ldb_obj.OnTooltipShow(ldb_tip)

	elseif IsAltKeyDown() and not TheTravellersReferenceDB.drag then
		TheTravellersReferenceDB.full = not TheTravellersReferenceDB.full

		set_ldb_obj_text()

	elseif IsShiftKeyDown() and not TheTravellersReferenceDB.drag then
		TheTravellersReferenceDB.range = not TheTravellersReferenceDB.range

		set_ldb_obj_text()

	elseif not TheTravellersReferenceDB.drag then
		local func = ToggleWorldMap or ToggleFrame		
		func(WorldMapFrame)
	end
end

frame:RegisterEvent"PLAYER_LOGIN"
frame:RegisterEvent"ZONE_CHANGED"
frame:RegisterEvent"ZONE_CHANGED_INDOORS"
frame:RegisterEvent"ZONE_CHANGED_NEW_AREA"

frame:SetScript("OnEvent", set_ldb_obj_text)